@@ -25,7 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -63,6 +63,7 @@ static GdkEventMask x_grab_mask;
 static gboolean x_grab_owner_events;
 
 typedef UINT (WINAPI *t_WTInfoA) (UINT a, UINT b, LPVOID c);
+typedef UINT (WINAPI *t_WTInfoW) (UINT a, UINT b, LPVOID c);
 typedef BOOL (WINAPI *t_WTEnable) (HCTX a, BOOL b);
 typedef HCTX (WINAPI *t_WTOpenA) (HWND a, LPLOGCONTEXTA b, BOOL c);
 typedef BOOL (WINAPI *t_WTOverlap) (HCTX a, BOOL b);
@@ -70,6 +71,7 @@ typedef BOOL (WINAPI *t_WTPacket) (HCTX a, UINT b, LPVOID c);
 typedef int (WINAPI *t_WTQueueSizeSet) (HCTX a, int b);
 
 static t_WTInfoA p_WTInfoA;
+static t_WTInfoW p_WTInfoW;
 static t_WTEnable p_WTEnable;
 static t_WTOpenA p_WTOpenA;
 static t_WTOverlap p_WTOverlap;
@@ -334,7 +336,8 @@ _gdk_input_wintab_init_check (void)
   AXIS axis_x, axis_y, axis_npressure, axis_or[3];
   int i, k;
   int devix, cursorix;
-  char devname[100], csrname[100];
+  wchar_t devname[100], csrname[100];
+  gchar *devname_utf8, *csrname_utf8;
   BOOL defcontext_done;
   HMODULE wintab32;
 
@@ -353,6 +356,8 @@ _gdk_input_wintab_init_check (void)
 
   if ((p_WTInfoA = (t_WTInfoA) GetProcAddress (wintab32, "WTInfoA")) == NULL)
     return;
+  if ((p_WTInfoW = (t_WTInfoW) GetProcAddress (wintab32, "WTInfoW")) == NULL)
+    return;
   if ((p_WTEnable = (t_WTEnable) GetProcAddress (wintab32, "WTEnable")) == NULL)
     return;
   if ((p_WTOpenA = (t_WTOpenA) GetProcAddress (wintab32, "WTOpenA")) == NULL)
@@ -395,13 +400,17 @@ _gdk_input_wintab_init_check (void)
     {
       LOGCONTEXT lc;
       
-      /* We open the Wintab device (hmm, what if there are several?) as a
-       * system pointing device, i.e. it controls the normal Windows
+      /* We open the Wintab device (hmm, what if there are several, or
+       * can there even be several, probably not?) as a system
+       * pointing device, i.e. it controls the normal Windows
        * cursor. This seems much more natural.
        */
 
-      (*p_WTInfoA) (WTI_DEVICES + devix, DVC_NAME, devname);
-      
+      (*p_WTInfoW) (WTI_DEVICES + devix, DVC_NAME, devname);
+      devname_utf8 = g_utf16_to_utf8 (devname, -1, NULL, NULL, NULL);
+#ifdef DEBUG_WINTAB
+      GDK_NOTE (INPUT, (g_print("Device %d: %s\n", devix, devname_utf8)));
+#endif
       (*p_WTInfoA) (WTI_DEVICES + devix, DVC_NCSRTYPES, &ncsrtypes);
       (*p_WTInfoA) (WTI_DEVICES + devix, DVC_FIRSTCSR, &firstcsr);
       (*p_WTInfoA) (WTI_DEVICES + devix, DVC_HARDWARE, &hardware);
@@ -433,7 +442,7 @@ _gdk_input_wintab_init_check (void)
       lc.lcOptions |= CXO_MESSAGES;
       lc.lcStatus = 0;
       lc.lcMsgBase = WT_DEFBASE;
-      lc.lcPktRate = 50;
+      lc.lcPktRate = 0;
       lc.lcPktData = PACKETDATA;
       lc.lcPktMode = PACKETMODE;
       lc.lcMoveMask = PACKETDATA;
@@ -484,7 +493,7 @@ _gdk_input_wintab_init_check (void)
       for (cursorix = firstcsr; cursorix < firstcsr + ncsrtypes; cursorix++)
 	{
 #ifdef DEBUG_WINTAB
-	      GDK_NOTE (INPUT, (g_print("Cursor %d:\n", cursorix), print_cursor (cursorix)));
+	  GDK_NOTE (INPUT, (g_print("Cursor %d:\n", cursorix), print_cursor (cursorix)));
 #endif
 	  active = FALSE;
 	  (*p_WTInfoA) (WTI_CURSORS + cursorix, CSR_ACTIVE, &active);
@@ -500,12 +509,14 @@ _gdk_input_wintab_init_check (void)
 	   * at least for Wacom, skip cursors with physid zero.
 	   */
 	  (*p_WTInfoA) (WTI_CURSORS + cursorix, CSR_PHYSID, &physid);
-	  if (strcmp (devname, "WACOM Tablet") == 0 && physid == 0)
+	  if (wcscmp (devname, L"WACOM Tablet") == 0 && physid == 0)
 	    continue;
 
 	  gdkdev = g_object_new (GDK_TYPE_DEVICE, NULL);
-	  (*p_WTInfoA) (WTI_CURSORS + cursorix, CSR_NAME, csrname);
-	  gdkdev->info.name = g_strconcat (devname, " ", csrname, NULL);
+	  (*p_WTInfoW) (WTI_CURSORS + cursorix, CSR_NAME, csrname);
+	  csrname_utf8 = g_utf16_to_utf8 (csrname, -1, NULL, NULL, NULL);
+	  gdkdev->info.name = g_strconcat (devname_utf8, " ", csrname_utf8, NULL);
+	  g_free (csrname_utf8);
 	  gdkdev->info.source = GDK_SOURCE_PEN;
 	  gdkdev->info.mode = GDK_MODE_SCREEN;
 	  gdkdev->info.has_cursor = TRUE;
@@ -602,6 +613,7 @@ _gdk_input_wintab_init_check (void)
 	  _gdk_input_devices = g_list_append (_gdk_input_devices,
 					      gdkdev);
 	}
+      g_free (devname_utf8);
     }
 }
 
@@ -635,7 +647,8 @@ gdk_input_translate_coordinates (GdkDevicePrivate *gdkdev,
 				 gdouble          *x_out,
 				 gdouble          *y_out)
 {
-  GdkWindowImplWin32 *impl, *root_impl;
+  GdkWindowImplWin32 *root_impl;
+  GdkWindowObject *window_object;
 
   int i;
   int x_axis = 0;
@@ -644,33 +657,31 @@ gdk_input_translate_coordinates (GdkDevicePrivate *gdkdev,
   double device_width, device_height;
   double x_offset, y_offset, x_scale, y_scale;
 
-  impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (input_window->window)->impl);
+  window_object = GDK_WINDOW_OBJECT (input_window);
 
   for (i=0; i<gdkdev->info.num_axes; i++)
     {
       switch (gdkdev->info.axes[i].use)
-	{
-	case GDK_AXIS_X:
-	  x_axis = i;
-	  break;
-	case GDK_AXIS_Y:
-	  y_axis = i;
-	  break;
-	default:
-	  break;
-	}
+		{
+		case GDK_AXIS_X:
+		  x_axis = i;
+		  break;
+		case GDK_AXIS_Y:
+		  y_axis = i;
+		  break;
+		default:
+		  break;
+		}
     }
   
-  device_width = gdkdev->axes[x_axis].max_value - 
-		   gdkdev->axes[x_axis].min_value;
-  device_height = gdkdev->axes[y_axis].max_value - 
-                    gdkdev->axes[y_axis].min_value;
+  device_width = gdkdev->axes[x_axis].max_value - gdkdev->axes[x_axis].min_value;
+  device_height = gdkdev->axes[y_axis].max_value - gdkdev->axes[y_axis].min_value;
 
   if (gdkdev->info.mode == GDK_MODE_SCREEN) 
     {
       root_impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (_gdk_root)->impl);
-      x_scale = root_impl->width / device_width;
-      y_scale = root_impl->height / device_height;
+      x_scale = GDK_WINDOW_OBJECT (_gdk_root)->width / device_width;
+      y_scale = GDK_WINDOW_OBJECT (_gdk_root)->height / device_height;
 
       x_offset = - input_window->root_x;
       y_offset = - input_window->root_y;
@@ -680,50 +691,48 @@ gdk_input_translate_coordinates (GdkDevicePrivate *gdkdev,
       double device_aspect = (device_height*gdkdev->axes[y_axis].resolution) /
 	(device_width*gdkdev->axes[x_axis].resolution);
 
-      if (device_aspect * impl->width >= impl->height)
-	{
-	  /* device taller than window */
-	  x_scale = impl->width / device_width;
-	  y_scale = (x_scale * gdkdev->axes[x_axis].resolution)
-	    / gdkdev->axes[y_axis].resolution;
+      if (device_aspect * window_object->width >= window_object->height)
+		{
+		  /* device taller than window */
+		  x_scale = window_object->width / device_width;
+		  y_scale = (x_scale * gdkdev->axes[x_axis].resolution) / gdkdev->axes[y_axis].resolution;
 
-	  x_offset = 0;
-	  y_offset = -(device_height * y_scale - 
-			       impl->height)/2;
-	}
+		  x_offset = 0;
+		  y_offset = -(device_height * y_scale - window_object->height) / 2;
+		}
       else
-	{
-	  /* window taller than device */
-	  y_scale = impl->height / device_height;
-	  x_scale = (y_scale * gdkdev->axes[y_axis].resolution)
-	    / gdkdev->axes[x_axis].resolution;
+		{
+		  /* window taller than device */
+		  y_scale = window_object->height / device_height;
+		  x_scale = (y_scale * gdkdev->axes[y_axis].resolution)
+			/ gdkdev->axes[x_axis].resolution;
 
-	  y_offset = 0;
-	  x_offset = - (device_width * x_scale - impl->width)/2;
-	}
+		  y_offset = 0;
+		  x_offset = - (device_width * x_scale - window_object->width) / 2;
+		}
     }
 
-  for (i=0; i<gdkdev->info.num_axes; i++)
+  for (i = 0; i < gdkdev->info.num_axes; i++)
     {
       switch (gdkdev->info.axes[i].use)
-	{
-	case GDK_AXIS_X:
-	  axis_out[i] = x_offset + x_scale*axis_data[x_axis];
-	  if (x_out)
-	    *x_out = axis_out[i];
-	  break;
-	case GDK_AXIS_Y:
-	  axis_out[i] = y_offset + y_scale*axis_data[y_axis];
-	  if (y_out)
-	    *y_out = axis_out[i];
-	  break;
-	default:
-	  axis_out[i] =
-	    (gdkdev->info.axes[i].max * (axis_data[i] - gdkdev->axes[i].min_value) +
-	     gdkdev->info.axes[i].min * (gdkdev->axes[i].max_value - axis_data[i])) /
-	    (gdkdev->axes[i].max_value - gdkdev->axes[i].min_value);
-	  break;
-	}
+		{
+		case GDK_AXIS_X:
+		  axis_out[i] = x_offset + x_scale * axis_data[x_axis];
+		  if (x_out)
+			*x_out = axis_out[i];
+		  break;
+		case GDK_AXIS_Y:
+		  axis_out[i] = y_offset + y_scale * axis_data[y_axis];
+		  if (y_out)
+			*y_out = axis_out[i];
+		  break;
+		default:
+		  axis_out[i] =
+			(gdkdev->info.axes[i].max * (axis_data[i] - gdkdev->axes[i].min_value) +
+			 gdkdev->info.axes[i].min * (gdkdev->axes[i].max_value - axis_data[i])) /
+			(gdkdev->axes[i].max_value - gdkdev->axes[i].min_value);
+		  break;
+		}
     }
 }
 
