@@ -27,11 +27,17 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include	"gtktipsquery.h"
-#include	"gtksignal.h"
-#include	"gtktooltips.h"
-#include	"gtkmain.h"
-#include	"gtkintl.h"
+#undef GTK_DISABLE_DEPRECATED
+
+#include "config.h"
+#include "gtktipsquery.h"
+#include "gtksignal.h"
+#include "gtktooltips.h"
+#include "gtkmain.h"
+#include "gtkmarshalers.h"
+#include "gtkintl.h"
+
+#include "gtkalias.h"
 
 
 
@@ -100,6 +106,7 @@ gtk_tips_query_get_type (void)
 	(GtkClassInitFunc) NULL,
       };
 
+      I_("GtkTipsQuery");
       tips_query_type = gtk_type_unique (gtk_label_get_type (), &tips_query_info);
     }
 
@@ -129,42 +136,42 @@ gtk_tips_query_class_init (GtkTipsQueryClass *class)
   class->widget_entered = gtk_tips_query_widget_entered;
   class->widget_selected = NULL;
 
-  gtk_object_add_arg_type ("GtkTipsQuery::emit_always", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_EMIT_ALWAYS);
-  gtk_object_add_arg_type ("GtkTipsQuery::caller", GTK_TYPE_WIDGET, GTK_ARG_READWRITE, ARG_CALLER);
-  gtk_object_add_arg_type ("GtkTipsQuery::label_inactive", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_LABEL_INACTIVE);
-  gtk_object_add_arg_type ("GtkTipsQuery::label_no_tip", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_LABEL_NO_TIP);
+  gtk_object_add_arg_type ("GtkTipsQuery::emit-always", GTK_TYPE_BOOL, GTK_ARG_READWRITE | G_PARAM_STATIC_NAME, ARG_EMIT_ALWAYS);
+  gtk_object_add_arg_type ("GtkTipsQuery::caller", GTK_TYPE_WIDGET, GTK_ARG_READWRITE | G_PARAM_STATIC_NAME, ARG_CALLER);
+  gtk_object_add_arg_type ("GtkTipsQuery::label-inactive", GTK_TYPE_STRING, GTK_ARG_READWRITE | G_PARAM_STATIC_NAME, ARG_LABEL_INACTIVE);
+  gtk_object_add_arg_type ("GtkTipsQuery::label-no-tip", GTK_TYPE_STRING, GTK_ARG_READWRITE | G_PARAM_STATIC_NAME, ARG_LABEL_NO_TIP);
 
   tips_query_signals[SIGNAL_START_QUERY] =
-    gtk_signal_new ("start_query",
+    gtk_signal_new (I_("start-query"),
 		    GTK_RUN_FIRST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTipsQueryClass, start_query),
-		    gtk_marshal_VOID__VOID,
+		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   tips_query_signals[SIGNAL_STOP_QUERY] =
-    gtk_signal_new ("stop_query",
+    gtk_signal_new (I_("stop-query"),
 		    GTK_RUN_FIRST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTipsQueryClass, stop_query),
-		    gtk_marshal_VOID__VOID,
+		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   tips_query_signals[SIGNAL_WIDGET_ENTERED] =
-    gtk_signal_new ("widget_entered",
+    gtk_signal_new (I_("widget-entered"),
 		    GTK_RUN_LAST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTipsQueryClass, widget_entered),
-		    gtk_marshal_VOID__OBJECT_STRING_STRING,
+		    _gtk_marshal_VOID__OBJECT_STRING_STRING,
 		    GTK_TYPE_NONE, 3,
 		    GTK_TYPE_WIDGET,
 		    GTK_TYPE_STRING,
 		    GTK_TYPE_STRING);
   tips_query_signals[SIGNAL_WIDGET_SELECTED] =
-    g_signal_new ("widget_selected",
+    g_signal_new (I_("widget-selected"),
                   G_TYPE_FROM_CLASS(object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET(GtkTipsQueryClass, widget_selected),
                   _gtk_boolean_handled_accumulator, NULL,
-                  gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED,
+                  _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED,
                   G_TYPE_BOOLEAN, 4,
                   GTK_TYPE_WIDGET,
                   G_TYPE_STRING,
@@ -246,11 +253,7 @@ gtk_tips_query_get_arg (GtkObject             *object,
 static void
 gtk_tips_query_destroy (GtkObject	*object)
 {
-  GtkTipsQuery *tips_query;
-
-  g_return_if_fail (GTK_IS_TIPS_QUERY (object));
-
-  tips_query = GTK_TIPS_QUERY (object);
+  GtkTipsQuery *tips_query = GTK_TIPS_QUERY (object);
 
   if (tips_query->in_query)
     gtk_tips_query_stop_query (tips_query);
@@ -262,8 +265,7 @@ gtk_tips_query_destroy (GtkObject	*object)
   g_free (tips_query->label_no_tip);
   tips_query->label_no_tip = NULL;
 
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 GtkWidget*
@@ -305,10 +307,10 @@ gtk_tips_query_set_caller (GtkTipsQuery   *tips_query,
     g_return_if_fail (GTK_IS_WIDGET (caller));
 
   if (caller)
-    gtk_widget_ref (caller);
+    g_object_ref (caller);
 
   if (tips_query->caller)
-    gtk_widget_unref (tips_query->caller);
+    g_object_unref (tips_query->caller);
 
   tips_query->caller = caller;
 }
@@ -341,7 +343,8 @@ gtk_tips_query_real_start_query (GtkTipsQuery *tips_query)
   
   g_return_if_fail (GTK_IS_TIPS_QUERY (tips_query));
   
-  tips_query->query_cursor = gdk_cursor_new (GDK_QUESTION_ARROW);
+  tips_query->query_cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (tips_query)),
+							 GDK_QUESTION_ARROW);
   failure = gdk_pointer_grab (GTK_WIDGET (tips_query)->window,
 			      TRUE,
 			      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -351,7 +354,7 @@ gtk_tips_query_real_start_query (GtkTipsQuery *tips_query)
 			      GDK_CURRENT_TIME);
   if (failure)
     {
-      gdk_cursor_destroy (tips_query->query_cursor);
+      gdk_cursor_unref (tips_query->query_cursor);
       tips_query->query_cursor = NULL;
     }
   gtk_grab_add (GTK_WIDGET (tips_query));
@@ -365,13 +368,14 @@ gtk_tips_query_real_stop_query (GtkTipsQuery *tips_query)
   gtk_grab_remove (GTK_WIDGET (tips_query));
   if (tips_query->query_cursor)
     {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
-      gdk_cursor_destroy (tips_query->query_cursor);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (tips_query)),
+				  GDK_CURRENT_TIME);
+      gdk_cursor_unref (tips_query->query_cursor);
       tips_query->query_cursor = NULL;
     }
   if (tips_query->last_crossed)
     {
-      gtk_widget_unref (tips_query->last_crossed);
+      g_object_unref (tips_query->last_crossed);
       tips_query->last_crossed = NULL;
     }
   
@@ -414,12 +418,12 @@ gtk_tips_query_emit_widget_entered (GtkTipsQuery *tips_query,
 		       NULL,
 		       NULL,
 		       NULL);
-      gtk_widget_unref (tips_query->last_crossed);
+      g_object_unref (tips_query->last_crossed);
       tips_query->last_crossed = NULL;
     }
   else if (widget && widget != tips_query->last_crossed)
     {
-      gtk_widget_ref (widget);
+      g_object_ref (widget);
       if (tdata || tips_query->emit_always)
 	  gtk_signal_emit (GTK_OBJECT (tips_query),
 			   tips_query_signals[SIGNAL_WIDGET_ENTERED],
@@ -427,7 +431,7 @@ gtk_tips_query_emit_widget_entered (GtkTipsQuery *tips_query,
 			   tdata ? tdata->tip_text : NULL,
 			   tdata ? tdata->tip_private : NULL);
       if (tips_query->last_crossed)
-	gtk_widget_unref (tips_query->last_crossed);
+	g_object_unref (tips_query->last_crossed);
       tips_query->last_crossed = widget;
     }
 }
@@ -465,7 +469,11 @@ gtk_tips_query_event (GtkWidget	       *widget,
 	pointer_window = NULL;
       event_widget = NULL;
       if (pointer_window)
-	gdk_window_get_user_data (pointer_window, (gpointer*) &event_widget);
+	{
+	  gpointer event_widget_ptr;
+	  gdk_window_get_user_data (pointer_window, &event_widget_ptr);
+	  event_widget = event_widget_ptr;
+	}
       gtk_tips_query_emit_widget_entered (tips_query, event_widget);
       event_handled = TRUE;
       break;
@@ -511,3 +519,6 @@ gtk_tips_query_event (GtkWidget	       *widget,
 
   return event_handled;
 }
+
+#define __GTK_TIPS_QUERY_C__
+#include "gtkaliasdef.c"
